
import { _decorator, Component, Node, MotionStreak, Color, Button, Prefab, instantiate } from 'cc';
import { IQScrollViewItem_SetData, QScrollViewItem } from './QScrollViewItem';
const { ccclass, property } = _decorator;

/**
 * Predefined variables
 * Name = TestView
 * DateTime = Tue Mar 01 2022 10:03:48 GMT+0800 (中国标准时间)
 * Author = IMI4197
 * FileBasename = TestView.ts
 * FileBasenameNoExtension = TestView
 * URL = db://assets/scripts/TestView.ts
 * ManualUrl = https://docs.cocos.com/creator/3.4/manual/zh/
 * 3.4.1 BUG反馈用测试类
 */

@ccclass('TestView')
export class TestView extends Component {
    /** 实例化的item父节点 */
    @property({ type: Node, tooltip: "实例化的item父节点" })
    itemParent: Node = null

    /** 跳转用预制体 */
    @property({ type: Prefab, tooltip: "跳转用预制体" })
    preItem: Prefab = null

    /** 当前打开的问题节点索引 */
    private _curQIndex: number = -1

    start() {
        let qDescArr: string[] = [
            "MotionStreak.color设置失败",
            "带有Widget组件的Button在active为true时 同一帧执行active=false active=true后无法响应点击"
        ]

        for (let i = 0; i < qDescArr.length; i++) {
            let item = instantiate(this.preItem)
            this.itemParent.addChild(item)
            let comp = item.getComponent(QScrollViewItem)
            comp.setClickCb(this._onQScrollViewItemClick.bind(this))
            comp.setData({ index: i, desc: qDescArr[i] })
        }
    }

    /**
     * 实例化的预制体被点击
     * @param data 
     */
    private _onQScrollViewItemClick(data: IQScrollViewItem_SetData) {
        let index = data.index
        this.updateQIndex(index)
    }

    /**
     * 更新需要显示的问题节点索引
     * @param index 
     * @returns 
     */
    updateQIndex(index: number) {
        if (this._curQIndex == index) {
            return
        }

        let children = this.node.children
        let openedNode = children[this._curQIndex]
        if (openedNode) {
            openedNode.active = false
        }
        this._curQIndex = index

        let waitToOpenNode = children[this._curQIndex]
        if (waitToOpenNode) {
            waitToOpenNode.active = true

            // 执行对应的测试方法
            if (this[`_q${this._curQIndex}`]) {
                this[`_q${this._curQIndex}`]()
            }
        }
    }

    /** 
     * MotionStreak.color设置失败
     * 报错：this._assembler.updateColor
     */
    private _q0() {
        let parent = this.node.children[0]
        let motion = parent.children[0]
        motion.getComponent(MotionStreak).color = Color.BLACK
    }

    //#region q1
    private _q1ButtonNode: Node = null

    /**
     * Button节点内如果有Widget组件 Button节点在同一帧执行active=false active=true 会导致节点无法监听点击事件
     * 如果不带Widget组件则无事 虽然正常情况下不会这么写逻辑 但窗体管理时可能会出现该情景
     */
    private _q1() {
        let parent = this.node.children[1]
        this._q1ButtonNode = parent.children[0]
    }

    q1_showButtonClick() {
        this._q1ButtonNode.active = true
    }

    q1_hideButtonClick() {
        this._q1ButtonNode.active = false
    }

    q1_hideAndShowButtonClick() {
        this._q1ButtonNode.active = false
        this._q1ButtonNode.active = true
    }
    //#endregion
}